/* $Revision: 1.9 $ */

/****************************************************************************
 *
 * PROJECT: X-Core / My Thesis
 *
 * MODULE:  Terrain Generators
 *
 * FILE:    XcGridDrv.C
 *
 * DESCRIPTION: Driver interface to readers and writers
 *	Each driver inherits this interface.
 *
 * EXPORTS:  
 *
 * NOTES:        
 *
 *
 * (c) Copyright 2004 Kurt Schwehr. All rights reserved.
 ***************************************************************************/


/***************************************************************************
 * INCLUDES
 ***************************************************************************/

//
// C headers
//
#include <cassert>
#include <cstdlib>

//
// C++ headers
//
#include <iostream>
#include <fstream>
#include <string>
#include <vector>

//
// Local headers
//

#include "XcGridDrv.H"

//////////////////////////////////////////////////////////////////////
// XcGridOpts Implementation
//////////////////////////////////////////////////////////////////////
#if 0
//constructor 
namespace XcG {
XcGridOpts::XcGridOpts () {
  //ascii = false;  // Want to write binary pgm preferably.
  maxVal = 255;	// Can be either 2^8-1 or 2^16-1 for pgm

  textureFile = ""; // No texture file by default
  cptFile = "";

  explicitNormals = false; // Have inventor calculate the normals for us.
  colorTable = false;
  normalSpikes = false;  // True is mostly for debugging.

  includeCamera = true; // Povray: include a camera (false: this is a file to be included)
}


void XcGridOpts::print () {
  std::cout << "XcGridOpts:" << std::endl
	    << "  maxVal: " << maxVal << std::endl
	    << "  textureFile: " << textureFile << std::endl
	    << "  cptFile: " << cptFile << std::endl
	    << "  explicitnormals: " << (explicitNormals?"true":"false") << std::endl
	    << "  colorTable: " << (colorTable?"true":"false") << std::endl
	    << "  normalSpikes: " << (normalSpikes?"true":"false") << std::endl
	    << "  includeCamera: " << (includeCamera?"true":"false") << std::endl
	    << std::endl;
}

} // namespace XcG
#endif


//////////////////////////////////////////////////////////////////////
// XcGridDrv Implementation
//////////////////////////////////////////////////////////////////////

//namespace XcG {

std::string XcG::XcGridDrv::getName() { return("Base driver");}
bool XcG::XcGridDrv::canReadType(FileType t) {return (false);}
bool XcG::XcGridDrv::canWriteType(FileType t) {return(false);}
std::vector<XcG::FileType> XcG::XcGridDrv::getReadTypes() {
  std::vector<FileType> types;  // Leave it empty
  return (types);
}
std::vector<XcG::FileType> XcG::XcGridDrv::getWriteTypes() {
  std::vector<FileType> types;  // Leave it empty
  return (types);
}

bool XcG::XcGridDrv::ReadGrid(std::string filename, FileType t, XcGrid &grd) {
  return (false); // Can't read nuthin.
}
bool XcG::XcGridDrv::WriteGrid(std::string filename, FileType t, XcGrid &grd,
			       gengetopt_args_info &opts)
{
  return (false); // Can't write nuthin.
}


//} // namespace XcG

// Driver registration used to be here.


void XcG::XcGridDrvDelete(VecDriverPtr &drivers) {
  size_t i;
  for (i=0;i<drivers.size();i++) {
    //std::cout << "Removing driver: " << drivers[i]->getName() << std::endl;
    delete drivers[i];
  }
  drivers.clear();
}

using namespace std;
// -1 of no driver... FIX: make this the cpp not found id thingy!!!!
size_t XcG::XcGridDrvGetReader (VecDriverPtr &drivers, XcG::FileType t) {
  size_t i;
  for (i=0;i<drivers.size();i++) {
    if (drivers[i]->canReadType(t)) return (i);
  }
  return (string::npos); // FIX:  hack!  What is the best way?
}

// -1 of no driver... FIX: make this the cpp not found id thingy!!!!
size_t XcG::XcGridDrvGetWriter (VecDriverPtr &drivers, XcG::FileType t) {
  size_t i;
  for (i=0;i<drivers.size();i++) {
    if (drivers[i]->canWriteType(t)) return (i);
  }
  return (string::npos); // FIX:  hack!  what is the best way?
}

void XcG::XcGridDrvPrint(XcG::VecDriverPtr &drivers) {
  std::cout << "  Listing registered drivers: " << std::endl;
  {
    size_t drv;
    for (drv=0;drv<drivers.size();drv++) {
      std::cout << "    Driver: " << drivers[drv]->getName() << std::endl;
      size_t t;
      vector<XcG::FileType> types = drivers[drv]->getReadTypes();
      for (t=0;t<types.size();t++)
	std::cout << "      Read : " << XcGridFileTypeString(types[t]) << std::endl;
      types = drivers[drv]->getWriteTypes();
      for (t=0;t<types.size();t++)
	std::cout << "      Write: " << XcGridFileTypeString(types[t]) << std::endl;
    }
  }
} // XcGridDrvPrint

//////////////////////////////////////////////////////////////////////
// gengetopt_args_info
//////////////////////////////////////////////////////////////////////

// Since gengetopt does not provide a separate initializer function
// we have to make out own.
//
// FIX: add --cxx -C to gengetopt to make this a c++ class

void
gengetopt_args_info_INIT (struct gengetopt_args_info &a) 
{
  a.help_given = 0 ;
  a.version_given = 0 ;
  a.in_given = 0 ;
  a.out_given = 0 ;
  a.format_given = 0 ;
  a.cpt_given = 0 ;
  a.tex_given = 0 ;
  a.list_given = 0 ;
  a.normals_given = 0 ;
  a.spikes_given = 0 ;
  a.xoffset_given = 0 ;
  a.yoffset_given = 0 ;
  a.zoffset_given = 0 ;
  a.zscale_given = 0 ;
  a.verbose_given = 0 ;
  a.in_arg = NULL;
  a.out_arg = NULL;
  a.format_arg = NULL;
  a.cpt_arg = NULL;
  a.tex_arg = NULL;
  a.normals_flag = 0;
  a.spikes_flag = 0;
  a.xoffset_arg = 0.0 ;
  a.yoffset_arg = 0.0 ;
  a.zoffset_arg = 0.0 ;
  a.zscale_arg = 1.0 ;
  a.verbose_arg = 0 ;
  return;
} // gengetopt_args_info_INIT

//////////////////////////////////////////////////////////////////////
// TEST CODE
//////////////////////////////////////////////////////////////////////

#ifdef REGRESSION_TEST

using namespace XcG;
using namespace std;

int main (int argc, char *argv[]) {
  bool ok=true;
  bool r; // result of each test.

  VecDriverPtr drivers = XcGridDrvRegister();  // vector of drivers
  r = (2<drivers.size()); // Should get at least 3 drivers (vicar, pds, pgm)
  std::cout << "  XcGridDrvRegister test: " << (ok?"ok.":": FAILED!") << std::endl;

  // PRINT
  XcGridDrvPrint(drivers);


  // FIX: what is the not found for vector, it's NOT npos??
  {
    size_t drv;
    drv = XcGridDrvGetReader(drivers,VICAR);
    if (string::npos==drv || !drivers[drv]->canReadType(VICAR)) {
      std::cout << "  ERROR:  vicar driver failed to respond correctly" << std::endl;
      ok=false;
    }
    drv = XcGridDrvGetWriter(drivers,VICAR);
    if (string::npos==drv || !drivers[drv]->canWriteType(VICAR)) {
      std::cout << "  ERROR:  vicar driver failed to respond correctly" << std::endl;
      ok=false;
    }
  }


  XcGridDrvDelete(drivers);
  r = (0==drivers.size());
  std::cout << "  XcGridDrvDelete test: " << (r?"ok.":": FAILED!") << std::endl;
  if (!r) ok=false;

  std::cout << argv[0] << (ok?": ok.":": FAILED!") << std::endl;
  return (ok?EXIT_SUCCESS:EXIT_FAILURE);
}
#endif // REGRESSION_TEST
