/* $Revision: 1.12 $ */

/****************************************************************************
 *
 * PROJECT: X-Core / My Thesis
 *
 * MODULE:  Terrain Generators
 *
 * FILE:    XcGridDrv.C
 *
 * DESCRIPTION: Driver interface to readers and writers
 *	Each driver inherits this interface.
 *
 * EXPORTS:  
 *
 * NOTES:        
 *
 *
 * (c) Copyright 2004 Kurt Schwehr. All rights reserved.
 ***************************************************************************/

// Include Guard
#ifndef __X_GRID_DRV_H_
#define __X_GRID_DRV_H_
/***************************************************************************
 * INCLUDES
 ***************************************************************************/

#include <string>
#include <vector>

#include "XcGridFile.H"
#include "cmdline.h"

class XcGrid;

// FIX: what about virtual constructor/destructor???

namespace XcG {
  // File type drivers base class
  class XcGridDrv {
  public:
    // FIX: why do we need an ID number?
    //virtual int getID() {return id;}
    //virtual void setID(int _id) {id=_id;}
    //int getID() {return id;}
    //void setID(int _id) {id=_id;}
    virtual std::string getName();
    virtual bool canReadType(const FileType t);
    virtual bool canWriteType(const FileType t);
    virtual std::vector<FileType> getReadTypes();
    virtual std::vector<FileType> getWriteTypes();
    // The ReadGrid does not do any of the generate functions for normals, triangles, etc.
    virtual bool ReadGrid(std::string filename, FileType t, XcGrid &grd);
    //virtual bool WriteGrid(std::string filename, FileType t, XcGrid &grd, XcGridOpts &opts);
    virtual bool WriteGrid(std::string filename, FileType t, XcGrid &grd,
			   gengetopt_args_info &opts);
  private:
    //int id;  // Can I remove this?
  };

  typedef std::vector<XcGridDrv *> VecDriverPtr;

  VecDriverPtr XcGridDrvRegister();
  void XcGridDrvDelete(VecDriverPtr &drivers);
  size_t XcG::XcGridDrvGetReader (VecDriverPtr &drivers, XcG::FileType t);
  size_t XcG::XcGridDrvGetWriter (VecDriverPtr &drivers, XcG::FileType t);
  void XcGridDrvPrint(VecDriverPtr &drivers);
} // namespace XcG

void gengetopt_args_info_INIT (struct gengetopt_args_info &a);


#endif // __X_GRID_DRV_H_
