/* $Revision: 1.13 $ */

/****************************************************************************
 *
 * PROJECT: X-Core / My Thesis
 *
 * MODULE:  Terrain Generators
 *
 * FILE:    XcGridPds.C
 *
 * DESCRIPTION: Fill in an XcGrid from a PDS file.  This is a JPL/MIPL
 * 	image format
 *
 * EXPORTS:  
 *
 * NOTES:        
 *   See the OS magic file
 *     Mac OS-X: /usr/share/file/magic
 *     Could also be /etc/magic
 *
 *     FIX: Change this to be a class since it needs initialization.
 *
 * (c) Copyright 2004 Kurt Schwehr. All rights reserved.
 ***************************************************************************/
  // FIX: add FITS http://fits.gsfc.nasa.gov/

/***************************************************************************
 * INCLUDES
 ***************************************************************************/

//
// C headers
//
#include <cassert>
#include <cstdlib>

//
// C++ headers
//
#include <iostream>
#include <fstream>
#include <string>
#include <map>

//
// Local headers
//

#include "XcGridFile.H"


using namespace XcG;
using namespace std;

typedef std::pair<std::string, FileType> FileTypeByNamePair;
typedef std::pair<FileType, std::string> FileTypeNamePair;
typedef std::map<std::string, FileType> FileTypeByNameMap;
typedef std::map<FileType, std::string> FileTypeNameMap;

FileTypeByNameMap fileTypeByName; // Indexed by string
FileTypeNameMap fileTypeName; // Indexed by enum


//bool FileTypeLUTInitialized=false;
void FileTypeLUTInit () {
  static bool FileTypeLUTInitialized=false;
  if (FileTypeLUTInitialized) return;
  fileTypeByName.insert(FileTypeByNamePair("UNKNOWN_FILETYPE",UNKNOWN_FILETYPE));
  fileTypeByName.insert(FileTypeByNamePair("BAD_FILE",BAD_FILE));

  fileTypeByName.insert(FileTypeByNamePair("ISUS",ISUS));
  fileTypeByName.insert(FileTypeByNamePair("PDS",PDS));
  fileTypeByName.insert(FileTypeByNamePair("VICAR",VICAR));

  fileTypeByName.insert(FileTypeByNamePair("JPG",JPG));
  fileTypeByName.insert(FileTypeByNamePair("GMT0",GMT0));

  fileTypeByName.insert(FileTypeByNamePair("PBM_BIN",PBM_BIN));
  fileTypeByName.insert(FileTypeByNamePair("PBM_ASCII",PBM_ASCII));

  fileTypeByName.insert(FileTypeByNamePair("PGM_BIN",PGM_BIN));
  fileTypeByName.insert(FileTypeByNamePair("PGM_ASCII",PGM_ASCII));

  fileTypeByName.insert(FileTypeByNamePair("PPM_BIN",PPM_BIN));
  fileTypeByName.insert(FileTypeByNamePair("PPM_ASCII",PPM_ASCII));

  fileTypeByName.insert(FileTypeByNamePair("TIF",TIF));
  fileTypeByName.insert(FileTypeByNamePair("PNG",PNG));
  fileTypeByName.insert(FileTypeByNamePair("GIF",GIF));

  fileTypeByName.insert(FileTypeByNamePair("IV",IV));
  fileTypeByName.insert(FileTypeByNamePair("POVRAY",POVRAY));
  fileTypeByName.insert(FileTypeByNamePair("VRML_ELEVATION_GRID",VRML_ELEVATION_GRID));

  fileTypeByName.insert(FileTypeByNamePair("NFF",NFF));
  fileTypeByName.insert(FileTypeByNamePair("E00",E00)); // ESRI export format

  // FIX: Only the FORM one may be valid
  fileTypeByName.insert(FileTypeByNamePair("LWO_FORM_IFF",LWO_FORM_IFF));
  fileTypeByName.insert(FileTypeByNamePair("LWO_SURF_IFF",LWO_SURF_IFF));
  fileTypeByName.insert(FileTypeByNamePair("LWO_POLS_IFF",LWO_POLS_IFF));
  fileTypeByName.insert(FileTypeByNamePair("LWO_LWOB_IFF",LWO_LWOB_IFF));

  fileTypeByName.insert(FileTypeByNamePair("OBJ",OBJ));


  {
    FileTypeByNameMap::iterator iter;
    for (iter=fileTypeByName.begin(); iter!= fileTypeByName.end(); iter++) {
      fileTypeName.insert(FileTypeNamePair(iter->second,iter->first));
    }
  }
} // FileTypeLUTInit


FileType XcG::XcGridGetFileType (std::string filename) {

  // FIX: Implement! First use the unix file command
  if (false) {
    ios::sync_with_stdio();
    //string cmd("file -b " + filename);
    //string cmd("file " + filename);
    FILE *inFile;
    if ( NULL == popen("ls","r")) {
      perror("popen failed");
      std::cerr << "popen failed\n";
      return (BAD_FILE);
    }
    const int MAXLINE=1024;
    char line[MAXLINE];
    while (NULL != fgets(line, MAXLINE, inFile)) {
      std::cout << line << "\n";
    }
    
    if (NULL==pclose(inFile)) {
      perror("pclose failed");
      std::cerr << "pclose failed\n";
      return (BAD_FILE);
    }
  } // END file

  std::ifstream in(filename.c_str(), ios::in|ios::binary|ios::ate);
  if (in.fail()) {
    std::cerr << "Failed to open file\n";
    return (BAD_FILE);
  }

  const size_t fileSize=in.tellg();
  //cout << "FileSize = " << fileSize << "\n";
  in.seekg(0);
  if (in.fail()) {
    std::cerr << "Failed to seek to start of file\n";
    return(BAD_FILE);
  }

  // Should be able to figure them all out from the first 1K
  const size_t MAXSIZE=1024;
  char *buf = new char[MAXSIZE+1];
  int bufSize;
  if (fileSize<MAXSIZE) bufSize=fileSize; else bufSize=MAXSIZE;
  in.read(buf,bufSize);
  if (in.fail()) {
    std::cerr << "ERROR: unable to read the file\n";
    delete [] buf;
    return (BAD_FILE);
  }

  // Now let's get out of this char* business.
  buf[bufSize]='\0'; // Make sure it's nul terminated
  string s(buf);
  //std::cout << s.length() << " length\n";

  size_t pos;
  string s2 = s.substr(0,2);
  string s3 = s.substr(0,3);
  string s4 = s.substr(0,4);
  string s10 = s.substr(0,10);
  //string s10 (10,buf);

  ////////////////////////////////////////
  //
  // WATCH OUT FOR MEMEORY LEAKS UNTIL THE delete [] buf
  {
    //http://www.w3.org/Graphics/JPEG/jfif.txt
    if (0xff==(unsigned char)buf[0] && 0xd8==(unsigned char)buf[1]) {
      delete [] buf;
      return (JPG);
    }
  }  // End of dangerous leak zone

  //
  // NOW GET RID OF THE C type data block.
  //
  delete [] buf;

  // FIX: what is ISUS??
  // Something like:
  // CCSD3ZF0000100000001NJPL3IF0PDS200000001 = SFDU_LABEL
  // eel:/Users/schwehr/Desktop/Archive/04Jan/rkirk-gusev/semifinal/gusevcratertopo.cub

  // Lightwave objects FORM, SURF, POLS, LWOB
  {
    //LWO_FORM_IFF, LWO_SURF_IFF, LWO_POLS_IFF, LWO_LWOB_IFF,
    pos = s4.find("FORM");
    if (string::npos != pos) {
      if (0==pos) return (LWO_FORM_IFF);
      else std::cerr<< "WARNING: almost thought this is LWO_FORM_IFF\n";
    }
    pos = s4.find("SURF");
    if (string::npos != pos) {
      if (0==pos) return (LWO_SURF_IFF);
      else std::cerr<< "WARNING: almost thought this is  LWO_SURF_IFF\n";
    }
    pos = s4.find("POLS");
    if (string::npos != pos) {
      if (0==pos) return (LWO_POLS_IFF);
      else std::cerr<< "WARNING: almost thought this is LWO_POLS_IFF\n";
    }
    pos = s4.find("LWOB");
    if (string::npos != pos) {
      if (0==pos) return (LWO_LWOB_IFF);
      else std::cerr<< "WARNING: almost thought this is  LWO_LWOB_IFF\n";
    }
  } // End Lightwave

  // NFF - World Toolkit "Neutral File Format"
  // NFF
  // FIX: how to do a case insensitive find?
  pos = s3.find("NFF");
  if (string::npos != pos) {
    if (0==pos) return (NFF);
    else std::cerr<< "WARNING: almost thought this is  NFF\n";
  }
  pos = s3.find("nff");
  if (string::npos != pos) {
    if (0==pos) return (NFF);
    else std::cerr<< "WARNING: almost thought this is  NFF\n";
  }

  // ESRI Export format
  pos = s3.find("EXP");
  if (string::npos != pos) {
    if (0==pos) return (E00);
    else std::cerr<< "WARNING: almost thought this is  E00\n";
  }

  // Handle version
  pos = s.find("PDS_VERSION_ID");
  if (string::npos != pos) {
    //std::cout << "Found PDS\n";
    if (0==pos) return (PDS);
    else std::cerr<< "WARNING: almost thought this is PDS\n";
  }

  pos = s.find("LBLSIZE=");
  if (string::npos != pos) {
    //std::cout << "Found Vicar\n  POS="<<pos<<"\n";
    if (0==pos) return (VICAR);
    else std::cerr<< "WARNING: almost thought this is VICAR\n";
  }

  // GMT1
  // PBM_BIN
  pos = s2.find("P4"); if (string::npos != pos) return (PBM_BIN);

  // PBM_ASCII
  pos = s2.find("P1");
  if (string::npos != pos) {
    if (0==pos) return (PBM_ASCII);
    else std::cerr<< "WARNING: almost thought this is PBM_ASCII\n";
  }
  // PGM_BIN
  pos = s2.find("P5");
  if (string::npos != pos) {
    if (0==pos) return (PGM_BIN);
    else std::cerr<< "WARNING: almost thought this is PGM_BIN\n";
  }
  // PGM_ASCII
  pos = s2.find("P2");
  if (string::npos != pos) {
    if (0==pos) return (PGM_ASCII);
    else std::cerr<< "WARNING: almost thought this is PGM_ASCII\n";
  }
  // PPM_BIN
  pos = s2.find("P6");
  if (string::npos != pos) {
    if (0==pos) return (PPM_BIN);
    else std::cerr<< "WARNING: almost thought this is PPM_BIN\n";
  }
  // PPM_ASCII
  pos = s2.find("P3");
  if (string::npos != pos) {
    if (0==pos) return (PPM_ASCII);
    else std::cerr<< "WARNING: almost thought this is PPM_ASCII\n";
  }

  // TIF
  pos = s2.find("MM"); // little endian
  if (string::npos != pos) return (TIF);
  pos = s2.find("II"); // big endian
  if (string::npos != pos) return (TIF);

  // PNG
  pos = s10.find("PNG"); // big endian
  if (string::npos != pos) return (PNG);

  // GIF
  pos = s3.find("GIF"); // big endian
  if (string::npos != pos) return (GIF);

  // GMT0 - NetCDF
  pos = s3.find("CDF"); // big endian
  if (string::npos != pos) return (GMT0);

  return (UNKNOWN_FILETYPE);
}

/***************************************************************************
 *
 * FUNCTION: FunctionName
 * DESCRIPTION: 
 * INPUTS:  
 * OUTPUTS: <What does the function generate other than the return?>
 * RETURNS: <What does the function return?>
 * EXCEPTIONS: 
 * DESIGN:  
 * NOTES:   
 *
 ***************************************************************************/

std::string XcG::XcGridFileTypeString (const FileType type) {
  FileTypeLUTInit();

  FileTypeNameMap::iterator iter;

  iter = fileTypeName.find(type);

  if (iter != fileTypeName.end()) {
    //std::cout << " found " << type << " " << iter->second << endl;
    return (iter->second);
  } else std::cout << "Did not find file type " << type << endl;
  
  return ("UNKNOWN_FILETYPE");
} // XcGridFileTypeString

/***************************************************************************
 *
 * FUNCTION: FunctionName
 * DESCRIPTION: 
 * INPUTS:  
 * OUTPUTS: <What does the function generate other than the return?>
 * RETURNS: <What does the function return?>
 * EXCEPTIONS: 
 * DESIGN:  
 * NOTES:   
 *
 ***************************************************************************/

XcG::FileType XcG::XcGridFileType (std::string typeName) {
  FileTypeLUTInit();
  FileTypeByNameMap::iterator iter;

  iter = fileTypeByName.find(typeName);

  if (iter != fileTypeByName.end()) {
    return (iter->second);
  } else std::cout << "Did not find file type " << typeName << endl;

  return (XcG::UNKNOWN_FILETYPE);
} //XcGridFileType 

//////////////////////////////////////////////////////////////////////
// TEST CODE
//////////////////////////////////////////////////////////////////////

#ifdef REGRESSION_TEST

#include <vector>

using namespace XcG;
using namespace std;

int main (int argc, char *argv[]) {
  bool ok=true;
  FileTypeLUTInit();
  {
    bool r=true;
    if (ISUS!=XcGridFileType("ISUS")) ok=r=false;
    if (!r) std::cout << "  ISUS lookup: FAILED!" << endl;

    if (PDS!=XcGridFileType("PDS")) ok=r=false;
    if (!r) std::cout << "  PDS lookup: FAILED!" << endl;

    if (VICAR!=XcGridFileType("VICAR")) ok=r=false;
    if (!r) std::cout << "  VICAR lookup: FAILED!" << endl;

    if (JPG!=XcGridFileType("JPG")) ok=r=false;
    if (!r) std::cout << "  JPG lookup: FAILED!" << endl;

    if (GMT0!=XcGridFileType("GMT0")) ok=r=false;
    if (!r) std::cout << "  GMT0 lookup: FAILED!" << endl;

    if (PGM_BIN!=XcGridFileType("PGM_BIN")) ok=r=false;
    if (!r) std::cout << "  PGM_BIN lookup: FAILED!" << endl;

    if (PGM_ASCII!=XcGridFileType("PGM_ASCII")) ok=r=false;
    if (!r) std::cout << "  PGM_ASCII lookup: FAILED!" << endl;

    if (IV!=XcGridFileType("IV")) ok=r=false;
    if (!r) std::cout << "  IV lookup: FAILED!" << endl;

    if (POVRAY!=XcGridFileType("POVRAY")) ok=r=false;
    if (!r) std::cout << "  POVRAY lookup: FAILED!" << endl;

    if (VRML_ELEVATION_GRID!=XcGridFileType("VRML_ELEVATION_GRID")) ok=r=false;
    if (!r) std::cout << "  VRML_ELEVATION_GRID lookup: FAILED!" << endl;
    // Doesn't actually check them all.
    // FIX: get lists from map and check forward against reverse lookups
  }

  std::vector<std::string> files;
  std::vector<FileType> types;

  files.push_back("TestData/2N126543636ETH0100P1505L0M1.IMG"); types.push_back(PDS);
  files.push_back("TestData/gen4x4byteLinux.vic"); types.push_back(VICAR);
  files.push_back("TestData/gray-4x4-mac-jpeg.tif"); types.push_back(TIF);
  files.push_back("TestData/gray-4x4.gif"); types.push_back(GIF);
  files.push_back("TestData/gray-4x4.jpg"); types.push_back(JPG);
  files.push_back("TestData/gray-4x4.png"); types.push_back(PNG);
  files.push_back("TestData/ascii.pgm"); types.push_back(PGM_ASCII);
  files.push_back("TestData/bin.pgm"); types.push_back(PGM_BIN);
  files.push_back("TestData/eel-em1000-subsample5.grd"); types.push_back(GMT0);
  //files.push_back("TestData/eel-em1000-subsample5.grd"); types.push_back(GMT1);

  size_t i=0;
  for (i=0;i < files.size();i++) {
    bool r=true;
    std::string s = files[i];
    FileType t = XcGridGetFileType (s);
    if (t != types[i]) {
      r=ok=false;
    }
    std::cout << "  " << files[i] << "  " << XcGridFileTypeString(t)
	      << (r?" ok.":": FAILED!") << std::endl;
  }

  std::cout << argv[0] << (ok?": ok.":": FAILED!") << std::endl;
  return (ok?EXIT_SUCCESS:EXIT_FAILURE);
}
#endif // REGRESSION_TEST
