
/****************************************************************************
 * $Revision: 1.3 $ $Author: schwehr $ $Date: 2004/05/15 23:54:40 $
 *
 * PROJECT: XCore
 *
 * MODULE: 
 *
 * FILE: 
 *
 * DESCRIPTION: 
 *
 * NOTES:
 *     man makecpt from GMT
 *     This version lets ranges overlap.  This first matching range wins.
 *
 * (c) Copyright 2003 Kurt Schwehr.  All rights reserved.
 * Also released under the GNU Public License.
 ****************************************************************************/


/****************************************************************************
 * INCLUDES
 ****************************************************************************/
#include <cassert>
#include <cstdlib>
#include <cmath>
#include <cstdio>	// sprintf()
//#include <unistd.h>	// getcwd()
#include <cstring>

#include <iostream>
#include <fstream>

#include <vector>
//#include <algo>
//#include <algobase>

using namespace std;

#include "colorPalette.H"

//////////////////////////////////////////////////////////////////////
// CPTentry
//////////////////////////////////////////////////////////////////////

bool
CPTentry::inRange(const float d) {
  if (d<depth[0]) return(false);
  if (d>depth[1]) return(false);
  return(true);
}

void
CPTentry::print() {
  cout << "CPTentry\n"
       << " depth   r   g   b\n"
       << "  " << depth[0] << " " 
       << minColor[0] << " " << minColor[1] << " " << minColor [2] << "\n"
       << "  " << depth[1] << " " 
       << maxColor[0] << " " << maxColor[1] << " " << maxColor [2] << "\n";
}

//////////////////////////////////////////////////////////////////////
// Color table 
//////////////////////////////////////////////////////////////////////



colorPalette::colorPalette() {
  //   depth[0]=-1.;  depth[1]=-1.;
  //   c1[0]=-1.;  c1[1]=-1.; c1[2]=-1.;
  //   c2[0]=-1.;  c2[1]=-1.; c2[2]=-1.;
  // FIX: NOOP?
}

// READ IN A CPT FILE.
bool
colorPalette::setColorsFromFile(const char *filename) {
  assert(filename);
  table.clear();  // Make sure there is nothing in the table.
  FILE *in=fopen(filename,"r");
  if (!in) {perror("could not load cpt");return(false);}
  bool hsvMode=false;
  char buf[1024];
  while (fgets(buf,1024,in)) {
    // Skip comments and look for HSV color mode.
    if ('#'==buf[0]) {
      //if (0!=strcasestr(buf,"COLOR_MODEL")) {
      //if (0!=strcasestr(buf,"HSV")) {
      if (0!=strstr(buf,"COLOR_MODEL")) {
	if (0!=strstr(buf,"HSV")) {
	  cerr << "setting HSV mode!!  may not work yet...\n";
	  hsvMode=true;
	}
      }
      continue;
    }
    float v[8];
    if (8!=sscanf(buf,"%f %f %f %f %f %f %f %f",
		  &v[0],&v[1],&v[2],&v[3],
		  &v[4],&v[5],&v[6],&v[7]
		  )) {
      cerr << "WARNING: bad cpt line... did not find 8 numbers:\n  "
	   << buf << "\n";
      continue;
    }
    if (!setRangeLinear(v[0],v[4],
			v[1],v[2],v[3],
			v[5],v[6],v[7])) {
      cerr << "WARNING: setRangeLinear failed for:\n  " << buf << "\n";
    }
  } // while fgets.
  return (true);
} // setColorsFromFile()


// depth 1 must be lower than depth 2
bool
colorPalette::setRangeLinear(const float depth1, const float depth2, 
			     const float r1, const float g1, const float b1,
			     const float r2, const float g2, const float b2
			     )


{
  assert ( 1.e6> depth1);
  assert (-1.e6< depth1);
  assert ( 1.e6> depth2);
  assert (-1.e6< depth2);
  assert (depth1<depth2);

  assert (255.0 >= r1);
  assert (0.0 <= r1);

  assert (255.0 >= g1);
  assert (0.0 <= g1);

  assert (255.0 >= b1);
  assert (0.0 <= b1);

  assert (255.0 >= r2);
  assert (0.0 <= r2);

  assert (255.0 >= g2);
  assert (0.0 <= g2);

  assert (255.0 >= b2);
  assert (0.0 <= b2);

  //   depth[0] = depth1;
  //   depth[1] = depth2;
  //   c1[0] = r1;
  //   c1[1] = g1;
  //   c1[2] = b1;
  //   c2[0] = r2;
  //   c2[1] = g2;
  //   c2[2] = b2;

  CPTentry c;
  c.depth[0]=depth1;
  c.depth[1]=depth2;
  c.minColor[0] = r1;
  c.minColor[1] = g1;
  c.minColor[2] = b1;
  c.maxColor[0] = r2;
  c.maxColor[1] = g2;
  c.maxColor[2] = b2;

  table.push_back(c);
  return (true);
}

// Linear translation of ranges.
float
translateRanges (const float val,
		 const float minIn, const float maxIn,
		 const float minOut, const float maxOut)
{
  const float rangeIn = maxIn - minIn;
  const float offIn = val - minIn;

  const float rangeOut = maxOut - minOut;
  const float offOut = minOut + (offIn/rangeIn) * rangeOut;
  

  return (offOut);
}
// FIX: write test routine for translate



bool
colorPalette::getColor(const float curDepth, float &r, float &g, float &b) {
  size_t i;
  bool found=false;
  for (i=0;i<table.size()&&!found;i++) {
    if (table[i].inRange(curDepth)) {
      found=true;
      break;
    }
  }

  if (!found) {
    static bool warned=false;
    if (!warned) {
      warned = true;
      cerr << "WARNING: unable to find depth in cpt: " << curDepth << "\n";
    }
    r = g = b = -1.;
    return (false);
  }

  CPTentry c = table[i];

  //c.print();

  r=translateRanges(curDepth,c.depth[0],c.depth[1],c.minColor[0],c.maxColor[0]);
  g=translateRanges(curDepth,c.depth[0],c.depth[1],c.minColor[1],c.maxColor[1]);
  b=translateRanges(curDepth,c.depth[0],c.depth[1],c.minColor[2],c.maxColor[2]);

  return (true);
}


// End Color table
//////////////////////////////////////////////////////////////////////


#ifdef REGRESSION_TEST
int
main (int argc, char *argv[]) {
  colorPalette cpt;
  cpt.setColorsFromFile("TestData/sample.cpt");

  bool ret,ok=true;
  float r,g,b,d;
  d=-100.0;ret=cpt.getColor (d,r,g,b); // cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";
  if (!ret) ok=false;

  d=+0000.1;ret=cpt.getColor (d,r,g,b); // cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";
  if (!ret) ok=false;
  d=+1000.0;ret=cpt.getColor (d,r,g,b); // cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";
  if (!ret) ok=false;
  d=+1500.0;ret=cpt.getColor (d,r,g,b); // cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";
  if (!ret) ok=false;

  d=+3000.0;ret=cpt.getColor (d,r,g,b); // cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";
  if (!ret) ok=false;
  d=+4000.0;ret=cpt.getColor (d,r,g,b); // cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";
  if (!ret) ok=false;
  d=+5000.0;ret=cpt.getColor (d,r,g,b); // cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";
  if (!ret) ok=false;
  d=+6000.0;ret=cpt.getColor (d,r,g,b); // cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";
  if (!ret) ok=false;
  d=+7000.0;ret=cpt.getColor (d,r,g,b); // cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";
  if (!ret) ok=false;
  d=+8000.0;ret=cpt.getColor (d,r,g,b); // cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";
  if (!ret) ok=false;
  d=+9000.0;ret=cpt.getColor (d,r,g,b); // cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";
  if (!ret) ok=false;
  d=10000.0;ret=cpt.getColor (d,r,g,b); //// cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";
  if (!ret) ok=false;

  // This one should fail
  d=+40000.0;
  cout << "   Expect a warning here about depth of "<<d<<":" << endl;
  if (cpt.getColor (d,r,g,b)) {
    //cerr << "should have returned false... out of range" << endl;
    ok=false;
  }
  //// cout<<d<<" "<<r<<" "<<g<<" "<<b<<"\n";

  // Test default copy constructor
  {
    colorPalette cpt2 = cpt;
    float r2,g2,b2;
    d=+1000.0;ret=cpt.getColor (d,r,g,b); if (!ret) ok=false;
    ret=cpt2.getColor(d,r2,g2,b2); if (!ret) ok=false;
    //cout << r << " " <<r2<<endl<<g<<" "<<g2<<endl<< b << " " << b2 << endl;
    cout << "   copy constructor: " ;
    if (r!=r2 || g!=g2 || b!=b2) {
      ok=false;
      cout << "FAILED!" << endl;
    }
    cout << "ok." << endl;
  }



  std::cout << argv[0] << (ok?": ok.":": FAILED!") << std::endl;
  return (ok?EXIT_SUCCESS:EXIT_FAILURE);
}

#endif //Regression test
