// #include <sys/types.h>
// #include <sys/stat.h>
// #include <fcntl.h> 	// open
// #include <sys/mman.h>	// mmap

// #include <cmath> // Why does it undefine isnan?
// extern "C" int isnan (double);  // FIX: what is the right way to do this???

#include <iostream>
#include <fstream>
#include <vector>
#include <string>

#include "lw-tags.H"
#include "lw-io.H"

using namespace std;


//////////////////////////////////////////////////////////////////////
// GLOBAL VARS... not good.
//////////////////////////////////////////////////////////////////////

static LwTag lwTag; // FIX: only want one copy of this thing floating around!

// Look for CLIP->STIL and return the string
// returns true is found, false if not.
// Does not handle the other types of STILs
bool GetTextureFile (LwFile &l, string &textureFile) {
  LwChunkClip    *clip = (LwChunkClip   *)l.getChunkPtr(CLIP);        if (!clip) { return(false); }
  LwSubChunkStil *stil = (LwSubChunkStil*)clip->getSubChunkPtr(STIL); if (!stil) { return(false); }
  textureFile = stil->name;
  return(true);
}

string StripPath(string s) {
  size_t slash = s.find_last_of("/");
  if (string::npos != slash)
    return (s.substr(slash+1,s.length()));
  cout << "FIX: check for ':' and strip that too." << endl;
  return(s);
}


//////////////////////////////////////////////////////////////////////
// WRITE OPEN INVENTOR CODE
//////////////////////////////////////////////////////////////////////

bool WriteInventor (const string filename, LwFile &l) {
  cout << "Write Inventor" << endl;
  bool ok=true;

  std::ofstream o(filename.c_str(), std::ios::out);
  if (!o.is_open()) {
    std::cerr << "ERROR: output file open failed for " << filename << endl;
    return (false);
  }

  o << "#Inventor V2.1 ascii" << endl << endl;

  // FIX: set creaseAngle to 
  o << "ShapeHints {" << endl
    << "vertexOrdering COUNTERCLOCKWISE" << endl
    << "  shapeType UNKNOWN_SHAPE_TYPE" << endl
    << "  faceType UNKNOWN_FACE_TYPE" << endl
    << "  creaseAngle 1.5" << endl
    << "}" << endl;

  //
  // Setup output stream floating point writing.  Want best precision.
  // 
  o.setf (std::ios_base::fixed,std::ios_base::floatfield);
  const int precision=12;  // FIX: what is really a good number for this?
  o.precision (precision);

  ////////////////////////////////////////
  // TEXTURE MAP
  ////////////////////////////////////////

  {
    string textureFileName;
    if (GetTextureFile (l, textureFileName)) {
      textureFileName=StripPath(textureFileName);
      cout << "found tex:  " << textureFileName << endl;
      o << "  Texture2 { filename \"" << textureFileName << "\" } " << endl;
      
    }
  }

  ////////////////////////////////////////
  // COORDINATES
  ////////////////////////////////////////
  {
    LwChunkPnts *pnts = (LwChunkPnts *)(l.getChunkPtr(PNTS));
    if (!pnts) {
      cerr << "WriteInventor ERROR: failed to find PNTS Chunk in lwo" << endl;
      return (false); 
    }
    if (PNTS != pnts->tag) {
      cerr << "ERROR: lier, lier, pants on fire!" << endl
	   << "  Chunk pretending to be PNTS Chunk" << lwTag.getString(pnts->tag) << endl;
      return (false);
    }

    o << "  # Num points = " << pnts->point_location.size() << endl;
    // FIX: should dynamically control precision based on the size of the number?
    //o.precision (precision); // Reset to higher precission
    o << "  Coordinate3 { point [\n";

    size_t i;
    for (i=0;i<pnts->point_location.size();i++) {
      VEC12 p = pnts->point_location[i];
      o << "      " << p.X << " " << p.Y << " " << p.Z << ",\n";
    }
    o << "    ] # point\n"
      << "  } # Coord3\n\n";
  } // Coords


  ////////////////////////////////////////
  // INDEX TO COORDS
  ////////////////////////////////////////
  {
    o << "  IndexedFaceSet { coordIndex [\n";
    LwChunkPols *pols = (LwChunkPols *)(l.getChunkPtr(POLS));
    if (!pols) {
      cerr << "WriteInventor ERROR: failed to find POLS Chunk in lwo" << endl;
      return (false); 
    }
    if (POLS != pols->tag) {
      cerr << "ERROR: lier, lier, pants on fire!" << endl
	   << "  Chunk pretending to be POLS Chunk" << lwTag.getString(pols->tag) << endl;
      return (false);
    }
    const size_t numPoly = pols->vertexSets.size();
    o << "    # Number of polys: "<< numPoly <<"\n";
    size_t p;
    for (p=0;p<numPoly;p++) {
      o << "    ";
      VertexSet vs = pols->vertexSets[p];
      size_t i;
      for (i=0;i<vs.vert.size();i++) o << vs.vert[i] << ",";
      o << "-1,\n";
    }
    o << "    ] # coordIndex\n"
      << "  } # IndexedFaceSet\n";
  } // Coord indexes


  


  o.close(); // FIX: error check... which one is right??? fail or bad?
  if (o.fail()) {
    ok=false;
    cerr << "ERROR(fail): close was not a happy camper for " << filename << endl;
  }
  if (o.bad()) {
    ok=false;
    cerr << "ERROR(bad): close was not a happy camper for " << filename << endl;
  }

  return (ok);
}

//////////////////////////////////////////////////////////////////////
// WRITE OPEN INVENTOR CODE
//////////////////////////////////////////////////////////////////////
int main (int argc, char *argv[]) {
  bool ok=true;
  bool r; // results from an individual test


  string filename(string("California.lwo")); // Looks pretty good now
  //string filename(string("../TestData/Triangle.lwo"));
  //string filename(string("../TestData/TriangleWithTexture.lwo"));
  //string filename(string("Zareh/McKittrick_nm_crop2-Morrored-reduced2.lwo"));
  r=false;
  LwFile lwFile(filename,r);
  if (!r) {
    cerr << "ERROR: LwFile constructor failed loading file: " << filename << endl;
    ok=false;
  }
  //cout << endl << "   TREE" << endl << endl;
  cout << endl << endl;
  lwFile.printTree();

  lwFile.printTreeFull();

  {
    LwChunk *bbox = lwFile.getChunkPtr(BBOX);
    bbox->print();
  }
  string outfile("foo.iv");
  r = WriteInventor(outfile, lwFile);
  if (!r) {
    ok=false;
    cerr << "ERROR: WriteInventor failed" << endl;
  }

  std::cout << argv[0] << (ok?": ok.":": FAILED!") << std::endl;
  return (ok?EXIT_SUCCESS:EXIT_FAILURE);
}
