
#include <ctype.h> // isprint()
#include <cstring>

#include <iostream>
#include <string>
using namespace std;

#include "xc-lightwave.H"


// Output:  the tag type and the tagLength
TagsEnum GetTagInfo (const char *buf, const size_t len, U4 &tagLength) {

  if (len<8) {
    cerr << "GetTag given less than 4 bytes" << endl;
    return (UNKNOWN_TAG);
  }
  tagLength = getU4 (buf+4,len-4);

#if 0
  // FIX: debugging yuck -- call printBuf instead
  {
    U4 i;
    cout << "GetTagInfo: " << tagLength + 8 << " len:" << len << endl;
    //for (i=0; i<len && i<8; i++) {
    // len+4 includes the 4 bytes of the length
    for (i=0; i<len+4 && i<tagLength+8; i++) {
      cout << i << "  " << (isprint(buf[i])?buf[i]:'*')
	   << " Int: " << int(buf[i]) << endl;
    }
  }
#else
  printBuf(buf,tagLength+8);
#endif

  // FIX: do something smarter
  if (0==strncmp("LAYR",buf,4)) return (LAYR);
  if (0==strncmp("PNTS",buf,4)) return (PNTS);
  if (0==strncmp("VMAP",buf,4)) return (VMAP);
  if (0==strncmp("POLS",buf,4)) return (POLS);
  if (0==strncmp("TAGS",buf,4)) return (TAGS);
  if (0==strncmp("PTAG",buf,4)) return (PTAG);
  if (0==strncmp("VMAD",buf,4)) return (VMAD);
  if (0==strncmp("ENVL",buf,4)) return (ENVL);
  if (0==strncmp("CLIP",buf,4)) return (CLIP);
  if (0==strncmp("SURF",buf,4)) return (SURF);
  if (0==strncmp("BBOX",buf,4)) return (BBOX);
  if (0==strncmp("DESC",buf,4)) return (DESC);
  if (0==strncmp("TEXT",buf,4)) return (TEXT);
  return (UNKNOWN_TAG);
}

string TagString (const TagsEnum tag) {
  switch (tag) {
  case LAYR: return (string("LAYR"));
  case PNTS: return (string("PNTS"));
  case VMAP: return (string("VMAP"));
  case POLS: return (string("POLS"));
  case TAGS: return (string("TAGS"));
  case PTAG: return (string("PTAG"));
  case VMAD: return (string("VMAD"));
  case ENVL: return (string("ENVL"));
  case CLIP: return (string("CLIP"));
  case SURF: return (string("SURF"));
  case BBOX: return (string("BBOX"));
  case DESC: return (string("DESC"));
  case TEXT: return (string("TEXT"));
  case UNKNOWN_TAG: return (string("UNKNOWN_TAG"));
  default:
    cerr << "TagString ERROR: Bad TAG!" << endl;
  }
  return (string("UNKNOWN_TAG"));
}



PolyEnum GetPolyEnum (const char *buf, const size_t bufSize) {
  assert(bufSize>=4);
  assert(buf);
  string s(buf,4);
  if (string::npos != s.find("FACE")) return (FACE);
  if (string::npos != s.find("CURV")) return (CURV);
  if (string::npos != s.find("PTCH")) return (PTCH);
  if (string::npos != s.find("MBAL")) return (MBAL);
  if (string::npos != s.find("BONE")) return (BONE);
  cerr << "ERROR: unable to determine polygon type" << endl;
  return (UNKNOWN_POLY_TYPE);
}


string PolyEnumString (const PolyEnum pe) {
  switch(pe) {
  case FACE: return(string("FACE"));
  case CURV: return(string("CURV"));
  case PTCH: return(string("PTCH"));
  case MBAL: return(string("MBAL"));
  case BONE: return(string("BONE"));
  case UNKNOWN_POLY_TYPE: return (string ("UNKNOWN_POLY_TYPE"));
  default:
    cerr << "ERROR: Bad Poly Enum String!!!" << endl;
  }
  return (string ("UNKNOWN_POLY_TYPE"));
}


// FIX: make a huge map of all enums and strings
// CLIP { index[U4], attributes[SUB-CHUNK] * }
ClipType GetClipType(const char *buf,size_t bufLen, const U4 tagLength) {
  cout << "Find out which type of CLIP this is" << endl;
  size_t offset=0;
  if (0==strncmp(buf,"CLIP",4)) offset=12;
  if (0==strncmp(buf+offset,"STIL",4)) return (STIL);
  if (0==strncmp(buf+offset,"ISEQ",4)) return (ISEQ);
  if (0==strncmp(buf+offset,"ANIM",4)) return (ANIM);
  if (0==strncmp(buf+offset,"XREF",4)) return (XREF);
  if (0==strncmp(buf+offset,"STCC",4)) return (STCC);

  return (UNKNOWN_CLIP_TYPE);
}

string ClipTypeString(const ClipType t) {
  switch (t) {
  case STIL: return (string("STIL"));
  case ISEQ: return (string("ISEQ"));
  case ANIM: return (string("ANIM"));
  case XREF: return (string("XREF"));
  case STCC: return (string("STCC"));
  case UNKNOWN_CLIP_TYPE: return (string("UNKNOWN_CLIP_TYPE"));
  default:
    cerr << "ERROR: bad clip type in ClipTypeString!!!" << endl;
  }
  return (string("BAD_CLIP_TYPE"));
}



SurfEnum getSurfEnum (const char *buf, size_t len) {
  assert (4<=len);
  if (0==strncmp("COLR",buf,4)) return (COLR);
  if (0==strncmp("DIFF",buf,4)) return (DIFF);
  if (0==strncmp("LUMI",buf,4)) return (LUMI);
  if (0==strncmp("SPEC",buf,4)) return (SPEC);
  if (0==strncmp("REFL",buf,4)) return (REFL);
  if (0==strncmp("TRAN",buf,4)) return (TRAN);
  if (0==strncmp("TRNL",buf,4)) return (TRNL);
  if (0==strncmp("GLOS",buf,4)) return (GLOS);
  if (0==strncmp("SHRP",buf,4)) return (SHRP);
  if (0==strncmp("BUMP",buf,4)) return (BUMP);
  if (0==strncmp("SIDE",buf,4)) return (SIDE);
  if (0==strncmp("SMAN",buf,4)) return (SMAN);
  if (0==strncmp("RFOP",buf,4)) return (RFOP);
  if (0==strncmp("RIMG",buf,4)) return (RIMG);

  if (0==strncmp("RSAN",buf,4)) return (RSAN);
  if (0==strncmp("RBLR",buf,4)) return (RBLR);
  if (0==strncmp("RIND",buf,4)) return (RIND);
  if (0==strncmp("TROP",buf,4)) return (TROP);
  if (0==strncmp("TIMG",buf,4)) return (TIMG);
  if (0==strncmp("TBLR",buf,4)) return (TBLR);
  if (0==strncmp("CLRH",buf,4)) return (CLRH);
  if (0==strncmp("CLRF",buf,4)) return (CLRF);
  if (0==strncmp("ADTR",buf,4)) return (ADTR);
  if (0==strncmp("GLOW",buf,4)) return (GLOW);
  if (0==strncmp("ALPH",buf,4)) return (ALPH);
  if (0==strncmp("VCOL",buf,4)) return (VCOL);

  if (0==strncmp("BLOK",buf,4)) return (BLOK);
  string s(buf,4);
  cerr << "Unknown surf tag: '" << s << "'" << endl;
  
  return (UNKNOWN_SURF_TAG);
}

string SurfEnumString(const SurfEnum t) {
  switch (t) {
  case COLR:  return (string("COLR"));
  case DIFF:  return (string("DIFF"));
  case LUMI:  return (string("LUMI"));
  case SPEC:  return (string("SPEC"));
  case REFL:  return (string("REFL"));
  case TRAN:  return (string("TRAN"));
  case TRNL:  return (string("TRNL"));
  case GLOS:  return (string("GLOS"));
  case SHRP:  return (string("SHRP"));
  case BUMP:  return (string("BUMP"));
  case SIDE:  return (string("SIDE"));
  case SMAN:  return (string("SMAN"));
  case RFOP:  return (string("RFOP"));
  case RIMG:  return (string("RIMG"));

  case RSAN:  return (string("RSAN"));
  case RBLR:  return (string("RBLR"));
  case RIND:  return (string("RIND"));
  case TROP:  return (string("TROP"));
  case TIMG:  return (string("TIMG"));
  case TBLR:  return (string("TBLR"));
  case CLRH:  return (string("CLRH"));
  case CLRF:  return (string("CLRF"));
  case ADTR:  return (string("ADTR"));
  case GLOW:  return (string("GLOW"));
  case ALPH:  return (string("ALPH"));
  case VCOL:  return (string("VCOL"));

  case BLOK:  return (string("BLOK"));
  default:
    cerr << "Unknown SURF enum" << endl;
  }

  return (string("UNKNOWN_SURF_ENUM error!"));
}

BlockEnum GetBlockEnum (const char *buf, const size_t buflen) {
  if (0==strncmp("IMAP",buf,4)) return (IMAP);
  if (0==strncmp("PROC",buf,4)) return (PROC);
  if (0==strncmp("GRAD",buf,4)) return (GRAD);
  if (0==strncmp("SHDR",buf,4)) return (SHDR);

  if (0==strncmp("CHAN",buf,4)) return (CHAN);
  if (0==strncmp("ENAB",buf,4)) return (ENAB);
  if (0==strncmp("OPAC",buf,4)) return (OPAC);
  if (0==strncmp("AXIS",buf,4)) return (AXIS);

  cerr << "ERROR: could not match BLOK type.  DRAT" << endl;
  return (UNKNOWN_BLOK_TYPE);
}

string BlockEnumString (const BlockEnum b) {
  switch (b) {
  case IMAP: return (string("IMAP"));
  case PROC: return (string("PROC"));
  case GRAD: return (string("GRAD"));
  case SHDR: return (string("SHDR"));

  case CHAN: return (string("CHAN"));
  case ENAB: return (string("ENAB"));
  case OPAC: return (string("OPAC"));
  case AXIS: return (string("AXIS"));
  default:
    cerr << "ERROR: Unknown block type!!!" << endl;
  }
  return (string("UNKNOWN_BLOK_TYPE error!"));
}
