

#include <sys/types.h>
#include <sys/stat.h>

#include <vector>


typedef uint32_t U4;
typedef uint16_t U2;
typedef uint8_t  U1;

typedef int32_t I4;
typedef int16_t I2;
typedef int8_t	I1;

typedef float  F4;
typedef string S0;

U4 getU4 (const char *buf, const size_t len);
U2 getU2 (const char *buf, const size_t len);
U1 getU1 (const char *buf, const size_t len);

I4 getI4 (const char *buf, const size_t len);
I2 getI2 (const char *buf, const size_t len);
I1 getI1 (const char *buf, const size_t len);

F4 getF4 (const char *buf, const size_t len);
string getID4 (const char *buf, const size_t len);
U4 getVX (const char *buf, const size_t len, size_t &size);


void printBuf (const char *buf, const size_t len);


enum PolyEnum {
  UNKNOWN_POLY_TYPE, FACE, CURV, PTCH, MBAL, BONE
};

enum TagsEnum {
  UNKNOWN_TAG,
  LAYR,
  PNTS,
  VMAP,
  POLS,
  TAGS, // string
  PTAG, // Polygon Tag Mapping
  VMAD, // Discont Vertex Mapping
  ENVL, // Envelope Definition
  CLIP, // Image or Image sequence
  SURF, // Surface Definition
  BBOX, 
  DESC, // Description S0
  TEXT,
  END_TAG
};

enum ClipType {
  STIL,  // Still Image
  ISEQ,  // Image Sequence
  ANIM,  // Plug-in Animation
  XREF,  // Reference (Clone)
  STCC,  // Color-cycling Still
  UNKNOWN_CLIP_TYPE  // Nothing we know about
};

enum SurfEnum {
  COLR,	// Base Color
  DIFF, LUMI, SPEC, REFL, TRAN, TRNL, // Basic Shading Values
  GLOS, // Specular Glossiness
  SHRP, // Diffuse Sharpness
  BUMP, // Bump Intensity
  SIDE, // Polygone Sideness
  SMAN, // Max Smoothing Angle
  RFOP, // Reflection Options
  RIMG, // Reflection Map Image
  RSAN, // Reflection Map Image Seam Angle
  RBLR, // Reflection blurring
  RIND, // Refractive Index
  TROP, // Transparency-options // same as in RFOP buf for refraction
  TIMG, // Refraction Map Image
  TBLR, // Refraction blurring
  CLRH, // Color Hightlights
  CLRF, // Color Filter
  ADTR, // Additive Transparency
  GLOW, // Glow Effect
  LINE, // Render outlines
  ALPH, // ALPHA mode
  VCOL, // Vertex Color Map

  BLOK, // Surface Blocks: BLOK { header[SUB-CHUNK],  attributes[SUB-CHUNK] * }

  UNKNOWN_SURF_TAG
};

enum BlockEnum {
  // HEADERS TYPES
  IMAP, // an image map texture
  PROC, // a procedural texture
  GRAD, // a gradient texture
  SHDR, // a shader plugin

  // HEADER LABELS
  CHAN, // Channel   ID4
  ENAB, // Enable state U2
  OPAC, // Opacity
  AXIS, // Displacement AXIS

  // Errr...
  NEGA,
  

  UNKNOWN_BLOK_TYPE
};

enum OpacityEnum {
  NORMAL=0,
  SUBTRACTIVE=1,
  DIFFERENCE=2,
  MULTIPLY=3,
  DIVIDE=4,
  ALPHA=5,
  TEXTURE_DISPLACEMENT=6,
  ADDITIVE=7,
  UNKNOWN_OPACITY=8
};




class coordinate {
public:
  F4 X,Y,Z;
  void getFromBuf(const char *buf,size_t bufLen);
  size_t getSizeOf(){return(4*3);}
  void print();
};

class bbox {
public:
  coordinate min,max;
};

class layer {
public:
  layer();
  void print();

  U2 number;
  U2 flags;
  coordinate pivot;
  string name;
  U2 parent;  // Index of the parent
};

class Polygon {
public:
  std::vector<size_t> indices;
};



class SurfaceSubChunk {
public:
  SurfaceSubChunk (const char *buf, size_t len);
  void print();

  SurfEnum type;
  char *start;
  U2 len;   // size of data, +6 to get whole thing including size and tag
};

class Surface {
public:
  char *start;
  U4 len;
  string name;
  string source;
  std::vector <SurfaceSubChunk> subchunks;
};

enum AxisEnum {
  X_AXIS=0,
  Y_AXIS=1,
  Z_AXIS=2,
  UNKNOWN_AXIS
};

class BlockHeader {
public:
  BlockHeader();
  void print();

  U2 CHAN_unknown;
  std::string CHAN_id;
  bool enabled; // ENAB
  OpacityEnum OPAC_type;
  float opacity; // [0..1]
  U4 envelope;

  AxisEnum displacement_axis;  // 0,1,2 for X, Y, Z respectively.

  //bool CHAN_given, ;
};


//////////////////////////////////////////////////////////////////////

// Output:  the tag type and the tagLength
TagsEnum GetTagInfo (const char *buf, const size_t len, U4 &tagLength);
string TagString (const TagsEnum tag);

string PolyEnumString (const PolyEnum pe);
PolyEnum GetPolyEnum (const char *buf, const size_t bufSize);

ClipType GetClipType(const char *buf,size_t bufLen, const U4 tagLength);
string ClipTypeString(const ClipType t);

SurfEnum getSurfEnum (const char *buf, size_t len);
string SurfEnumString(const SurfEnum t);

BlockEnum GetBlockEnum (const char *buf, const size_t buflen);
string BlockEnumString (const BlockEnum b);


