/* $Revision: 1.11 $ */

/****************************************************************************
 *
 * PROJECT: X-Core / My Thesis
 *
 * MODULE:  Terrain Generators
 *
 * FILE:    XcGridPgm.C
 *
 * DESCRIPTION: Use XcGrid to write/write pgm (pnm) format files.
 *
 * EXPORTS:  
 *
 * NOTES:        Starting off with only the writer
 *		FIX: convert this to using libnetpbm??
 *
 * (c) Copyright 2004 Kurt Schwehr. All rights reserved.
 ***************************************************************************/

//#include <cmath>
#include <iostream>
#include <fstream>
//using namespace std;

#include "XcGrid.H"
#include "XcGridDrv.H"
#include "xc-pbm.H"

/***************************************************************************
 * FUNCTION: XcGridWritePpmBinary
 * DESCRIPTION: PPM - color image binary writer
 * INPUTS:  
 * 	Max should be either 255 or 65535
 * OUTPUTS: 
 * RETURNS: true on success, false on failure
 * EXCEPTIONS: 
 * DESIGN:  
 * NOTES:   
 ***************************************************************************/
#include <vector>
using namespace std;

#include "colorPalette.H"

// old version... grd now contains the colortable
bool XcGridWritePpmBinary (const std::string filename, XcGrid &grd, const std::string cptFile, const size_t max) {
  assert (max < 65536);
  assert (255==max);  // FIX: deal with arbitrary size and figure if 1 or 2 byte
  std::ofstream o(filename.c_str(),std::ios::out);
  if (!o) {
    std::cerr << " output file open failed for "<< filename << "\n";
    return (false);
  }

  o << "P6\n"; // Magic Number

  // Write all of the log entries as comments
  {
    o << " # START LOG ENTRIES\n";
    o << " # NumLogEntries = " << grd.getNumLogEntries() << "\n";
    size_t i;
    for (i=0;i<grd.getNumLogEntries();i++) {
      o << " # "
	<< grd.getLogEntry(i)
	<< "\n";
    }
    o << "\n # END LOG ENTRIES\n\n";
  }

  // Image data starts after these thre integers...
  o << grd.getWidth() << " " << grd.getHeight() << "\n"
    << max << "\n";

  //float zmin = grd.getZMin();
  //float zmax = grd.getZMax();
  colorPalette cpt;
  cpt.setColorsFromFile(cptFile.c_str());
  // FIX: error check
  size_t x,y;
  for (y=0;y<grd.getHeight();y++) {
    for (x=0;x<grd.getWidth();x++) {
      // FIX: deal with max != 255
      const float z=grd.getZ(x,y);
      //float val=round( (z-zmin)/(zmax-zmin)  * 255 );
      float r,g,b;
      cpt.getColor (z,r,g,b);

      unsigned char cRed = (unsigned char)(r);//*max);
      unsigned char cGreen = (unsigned char)(g);//*max);
      unsigned char cBlue = (unsigned char)(b);//*max);
      o << cRed << cGreen << cBlue;
    }
  }
  o.close();

  return (true);
} // XcGridWritePpmBinary - PPM

/***************************************************************************
 * FUNCTION: XcGridWritePpmBinary
 * DESCRIPTION: PPM - color image binary writer
 * INPUTS:  
 * 	Max should be either 255 or 65535
 * OUTPUTS: 
 * RETURNS: true on success, false on failure
 * EXCEPTIONS: 
 * DESIGN:  
 * NOTES:   Grd contains the color table
 ***************************************************************************/

bool XcGridWritePpmBinary (const std::string filename, XcGrid &grd, const size_t max) {
  assert (max < 65536);
  assert (255==max);  // FIX: deal with arbitrary size and figure if 1 or 2 byte
  assert (grd.hasColorPalette());
  std::ofstream o(filename.c_str(),std::ios::out);
  if (!o) {
    std::cerr << " output file open failed for "<< filename << "\n";
    return (false);
  }

  o << "P6\n"; // Magic Number

  // Write all of the log entries as comments
  {
    o << " # START LOG ENTRIES\n";
    o << " # NumLogEntries = " << grd.getNumLogEntries() << "\n";
    size_t i;
    for (i=0;i<grd.getNumLogEntries();i++) {
      o << " # "
	<< grd.getLogEntry(i)
	<< "\n";
    }
    o << "\n # END LOG ENTRIES\n\n";
  }

  // Image data starts after these thre integers...
  o << grd.getWidth() << " " << grd.getHeight() << "\n" << max << "\n";

  colorPalette cpt = grd.getColorPalette();
  // FIX: error check
  size_t x,y;
  for (y=0;y<grd.getHeight();y++) {
    for (x=0;x<grd.getWidth();x++) {
      // FIX: deal with max != 255
      const float z=grd.getZ(x,y);
      float r,g,b;
      cpt.getColor (z,r,g,b);

      unsigned char cRed = (unsigned char)(r);//*max);
      unsigned char cGreen = (unsigned char)(g);//*max);
      unsigned char cBlue = (unsigned char)(b);//*max);
      o << cRed << cGreen << cBlue;
    }
  }
  o.close();
  return (true);
} // XcGridWritePpmBinary - PPM

/***************************************************************************
 * FUNCTION: XcGridWritePgmBinary
 * DESCRIPTION: 
 * INPUTS:  
 * 	Max should be either 255 or 65535
 * OUTPUTS: 
 * RETURNS: true on success, false on failure
 * EXCEPTIONS: 
 * DESIGN:  
 * NOTES:   
 ***************************************************************************/

bool
XcGridWritePgmBinary (const std::string filename, XcGrid &grd, const size_t max) {
  assert (max < 65536);
  assert (255==max);  // FIX: deal with arbitrary size and figure if 1 or 2 byte
  std::ofstream o(filename.c_str(),std::ios::out);
  if (!o) {
    std::cerr << " output file open failed for "<< filename << "\n";
    return (false);
  }

  o << "P5\n"; // Magic Number

  // Write all of the log entries as comments
  {
    o << " # START LOG ENTRIES\n";
    o << " # NumLogEntries = " << grd.getNumLogEntries() << "\n";
    size_t i;
    for (i=0;i<grd.getNumLogEntries();i++) {
      o << " # "
	<< grd.getLogEntry(i)
	<< "\n";
    }
    o << "\n # END LOG ENTRIES\n\n";
  }

  // Image data starts after these thre integers...
  o << grd.getWidth() << " " << grd.getHeight() << "\n"
    << max << "\n";

  float zmin = grd.getZMin();
  float zmax = grd.getZMax();

  size_t x,y;
  for (y=0;y<grd.getHeight();y++) {
    for (x=0;x<grd.getWidth();x++) {
      // FIX: deal with max != 255
      const float z=grd.getZ(x,y);
      float val=round( (z-zmin)/(zmax-zmin)  * 255 );
      unsigned char c = (unsigned char)val;
      o << c;
    }
  }
  o.close();

  return(true);
} // XcGridWritePgmBinary

/***************************************************************************
 * FUNCTION: XcGridWritePgmAscii
 * DESCRIPTION: 
 * INPUTS:  
 * 	Max should be either 255 or 65535
 * OUTPUTS: 
 * RETURNS: true on success, false on failure
 * EXCEPTIONS: 
 * DESIGN:  
 * NOTES:   
 ***************************************************************************/

bool
XcGridWritePgmAscii (const std::string filename, XcGrid &grd, const size_t max) {
  assert (max < 65536);
  assert (255==max);  // FIX: deal with arbitrary size and figure if 1 or 2 byte
  std::ofstream o(filename.c_str(),std::ios::out);
  if (!o) {
    std::cerr << " output file open failed for "<< filename << "\n";
    return (false);
  }

  o << "P2\n"; // Magic Number for pgm ascii

  // Write all of the log entries as comments
  {
    o << " # START LOG ENTRIES\n";
    o << " # NumLogEntries = " << grd.getNumLogEntries() << "\n";
    size_t i;
    for (i=0;i<grd.getNumLogEntries();i++) {
      o << " # "
	<< grd.getLogEntry(i)
	<< "\n";
    }
    o << "\n # END LOG ENTRIES\n\n";
  }

  // Image data starts after these thre integers...
  o << grd.getWidth() << " " << grd.getHeight() << "\n"
    << max << "\n";

  float zmin = grd.getZMin();
  float zmax = grd.getZMax();

  size_t x,y;
  int entries=0;
  for (y=0;y<grd.getHeight();y++) {
    for (x=0;x<grd.getWidth();x++) {
      // FIX: deal with max != 255
      const float z=grd.getZ(x,y);
      float val=round( (z-zmin)/(zmax-zmin)  * 255 );
      int i = (int)val;
      o << i << " ";
      entries++;
      if (entries > 10) {
	entries = 0;
	o << "\n";
      }
    }	
  }
  o.close();

  return(true);
} // XcGridWritePgmAscii

//////////////////////////////////////////////////////////////////////
// DRIVER IMPLEMENTATION - XcGridDrv
//////////////////////////////////////////////////////////////////////

std::string XcGridDrvPgm::getName() { return("PPM - ascii and binary");}
//bool XcGridDrvPgm::canReadType(FileType t) {return (false);}
//bool XcGridDrvPgm::canWriteType(FileType t) {return(false);}
std::vector<FileType> XcGridDrvPgm::getReadTypes() {
  std::vector<FileType> types;
  return (types);
}
std::vector<FileType> XcGridDrvPgm::getWriteTypes() {
  std::vector<FileType> types;
  types.push_back(PGM_ASCII);
  types.push_back(PGM_BIN);
  types.push_back(PPM_BIN);
  return (types);
}

bool XcGridDrvPgm::ReadGrid(std::string filename, FileType t, XcGrid &grd) {
  return (false); // Can't write nuthin.
}
bool XcGridDrvPgm::WriteGrid(std::string filename, FileType t, XcGrid &grd, gengetopt_args_info &opts) {
  //opts.print();
  const int maxVal=255; // FIX: make this configurable somehow.  Maybe with cpt?
  if (t==PGM_ASCII) {
    bool r;
    r=XcGridWritePgmAscii(filename,grd,maxVal);
    return (r);
  } 
  if (t==PGM_BIN) {
    bool r;
    r=XcGridWritePgmBinary(filename,grd,maxVal);
    return (r);
  } 
  if (t==PPM_BIN) {
    bool r;
    //r=XcGridWritePpmBinary(filename,grd,opts.cpt_arg,maxVal);
    // FIX: maybe great a default one somehow?
    if (!grd.hasColorPalette()) {
      std::cerr << "ERROR: PPM_BIN requires a color palette table" << std::endl;
      return(false);
    }
    r=XcGridWritePpmBinary(filename,grd,maxVal);
    return (r);
  } 
  std::cerr << "ERROR XcGridDrvPgm::WriteGrid: bad type." << std::endl;
  return (false);
}


//////////////////////////////////////////////////////////////////////
// TEST CODE
//////////////////////////////////////////////////////////////////////

#ifdef REGRESSION_TEST

//#include "xc-gmt.H"

int main (int argc, char *argv[]) {

#if 0
  {
    XcGrid g;
    XcGridGmtLoad ("mars.grd",g);
    XcGridWritePpmBinary("mars.ppm",g,"mars.cpt",255);
    std::cout << "EARLY exit FIX!!"<<std::endl;
    exit(EXIT_SUCCESS);
  }
#endif

  bool ok=true, r; // r is result for each test.
  {
    XcGrid g(2,2);
    g.setDelXDelY(1.0,1.0);
    g.setZ(0,0,0);
    g.setZ(1,0,100.1);
    g.setZ(0,1,99);
    g.setZ(1,1,255);

    r=XcGridWritePgmBinary("dummy-bin.pgm",g,255);
    std::cout << "  Write " << "dummy-bin.pgm" << (r?": ok.":": FAILED!")<<std::endl;
    if (!r) ok=false;

    r=XcGridWritePgmAscii ("dummy-ascii.pgm",g,255);
    std::cout << "  Write " << "dummy-ascii.pgm" << (r?": ok.":": FAILED!")<<std::endl;
    if (!r) ok=false;
  }


  std::cout << argv[0] << (ok?": ok.":": FAILED!")<<std::endl;
  return (ok?EXIT_SUCCESS:EXIT_FAILURE);
}
#endif // REGRESSION_TEST
