
/* $Revision: 1.7 $ */

/****************************************************************************
 *
 * PROJECT: X-Core / My Thesis
 *
 * MODULE:  Terrain Generators
 *
 * FILE:    VicarLabel.H
 *
 * DESCRIPTION: Read the label section of a vicar file into an STL Map
 *
 * EXPORTS:  
 *
 * NOTES:        
 *   - FIX: This should really be a subclass of VicarLabel!
 *
 * (c) Copyright 2004 Kurt Schwehr. All rights reserved.
 ***************************************************************************/

// Include Guard
#ifndef __PDSLABEL_H_
#define __PDSLABEL_H_

#include <string>
#include <vector>
#include <map>
#include <algorithm>

// http://www.codeproject.com/vcpp/stl/PracticalGuideStl.asp
// The above explains how to make a class that works with Maps

// FIX: should Reals be double?
class PdsLabel {
public:
  enum type {TYPE_UNKNOWN,POINTER,INT,REAL,STRING,INTARRAY,REALARRAY,STRARRAY};

  PdsLabel ();
  //PdsLabel (type t, std::string labelString);  // the whole string with =
  PdsLabel(type t, std::string labelString,
	   bool _inObj, int _objNum, std::string _objName);

  // Copy constructor
  PdsLabel(const PdsLabel &oldvc);
  void PdsLabel::operator= (const PdsLabel &oldvc);

  //
  // Generic labels
  //
  std::string getLabelName(); // return before the =
  std::string getLabelValStr(); // return after the =

  type getType() {return(labelType);}
  int getIntVal();
  float getRealVal();
  std::string getStringVal();
  std::string getWholeLabelStr(); // Return everything before and after the '='
  void print(); // For debugging

  // PDS Object info  (e.g. "IMAGE")
  bool isInObject() {return(inObject);}
  int getObjNum() {return(objNum);}
  std::string getObjName() {return(objName);}

private:
  std::string label;
  type labelType;
  int intVal;
  float realVal;
  std::string stringVal;
  std::vector<int> intArray;
  std::vector<float> realArray;
  std::vector<std::string> stringArray;

  // PDS Object info
  bool inObject;
  int objNum;
  std::string objName;
};

typedef std::map<std::string,PdsLabel> PdsLabelMap;
typedef std::pair<std::string,PdsLabel> PdsLabelPair;


// Return a map of all the labels.
PdsLabelMap LoadPdsLabelsFromFile(const std::string &filename,
				  std::vector<std::string> &objects);
void PrintLabels (PdsLabelMap &Labels);

const std::string NOT_IN_OBJECT="NOT IN OBJECT";


#endif // __PDSLABEL_H_
